@@ -198,6 +198,44 @@ socket file at FILE-NAME upon exit of PROC.  Return the values of PROC."
       ;; Start the 'root' service.
       (start root-service)
 
+      (when (= 1 (getpid))
+        ;; When running as PID 1, disable hard reboots upon ctrl-alt-del.
+        ;; Instead, the kernel will send us SIGINT so that we can gracefully
+        ;; shut down.  See ctrlaltdel(8) and kernel/reboot.c.
+        (catch 'system-error
+          (lambda ()
+            (disable-reboot-on-ctrl-alt-del))
+          (lambda args
+            (let ((err (system-error-errno args)))
+              ;; When in a separate PID namespace, we get EINVAL (see
+              ;; 'reboot_pid_ns' in kernel/pid_namespace.c.)  We get EPERM in
+              ;; a user namespace that lacks CAP_SYS_BOOT.
+              (unless (member err (list EINVAL EPERM))
+                (apply throw args)))))
+
+        ;; Load the SIGSEGV/SIGABRT handler.  This is what allows PID 1 to
+        ;; dump core on "/", should something go wrong.
+        (false-if-exception
+         (dynamic-link (string-append %pkglibdir "/crash-handler"))))
+
+      ;; Stop everything when we get SIGINT.
+      (sigaction SIGINT
+        (lambda _
+          (catch 'quit
+            (lambda ()
+              (stop root-service))
+            quit-exception-handler)))
+
+      ;; Stop everything when we get SIGTERM.
+      (sigaction SIGTERM
+        (lambda _
+          (stop root-service)))
+
+      ;; Stop everything when we get SIGHUP.
+      (sigaction SIGHUP
+        (lambda _
+          (stop root-service)))
+
       ;; This _must_ succeed.  (We could also put the `catch' around
       ;; `main', but it is often useful to get the backtrace, and
       ;; `caught-error' does not do this yet.)
@@ -226,39 +264,6 @@ socket file at FILE-NAME upon exit of PROC.  Return the values of PROC."
         (sigaction SIGALRM (lambda _ (alarm 1)))
         (alarm 1))
 
-      (when (= 1 (getpid))
-        ;; When running as PID 1, disable hard reboots upon ctrl-alt-del.
-        ;; Instead, the kernel will send us SIGINT so that we can gracefully
-        ;; shut down.  See ctrlaltdel(8) and kernel/reboot.c.
-        (catch 'system-error
-          (lambda ()
-            (disable-reboot-on-ctrl-alt-del))
-          (lambda args
-            (let ((err (system-error-errno args)))
-              ;; When in a separate PID namespace, we get EINVAL (see
-              ;; 'reboot_pid_ns' in kernel/pid_namespace.c.)  We get EPERM in
-              ;; a user namespace that lacks CAP_SYS_BOOT.
-              (unless (member err (list EINVAL EPERM))
-                (apply throw args))))))
-
-      ;; Stop everything when we get SIGINT.
-      (sigaction SIGINT
-        (lambda _
-          (catch 'quit
-            (lambda ()
-              (stop root-service))
-            quit-exception-handler)))
-
-      ;; Stop everything when we get SIGTERM.
-      (sigaction SIGTERM
-        (lambda _
-          (stop root-service)))
-
-      ;; Stop everything when we get SIGHUP.
-      (sigaction SIGHUP
-        (lambda _
-          (stop root-service)))
-
       ;; Ignore SIGPIPE so that we don't die if a client closes the connection
       ;; prematurely.
       (sigaction SIGPIPE SIG_IGN)
