@@ -1,5 +1,5 @@
 ;; service.scm -- Representation of services.
-;; Copyright (C) 2013, 2014, 2015, 2016, 2017, 2018, 2019 Ludovic Courtès <ludo@gnu.org>
+;; Copyright (C) 2013, 2014, 2015, 2016, 2017, 2018, 2019, 2020 Ludovic Courtès <ludo@gnu.org>
 ;; Copyright (C) 2002, 2003 Wolfgang Järling <wolfgang@pro-linux.de>
 ;; Copyright (C) 2014 Alex Sassmannshausen <alex.sassmannshausen@gmail.com>
 ;; Copyright (C) 2016 Alex Kost <alezost@gmail.com>
@@ -70,6 +70,8 @@
             provided-by
             required-by
             handle-unknown
+
+            default-environment-variables
             make-forkexec-constructor
             make-kill-destructor
             exec-command
@@ -485,9 +487,14 @@ is not already running, and will return SERVICE's canonical name in a list."
         (and (eq? root-service obj)
              (eq? key 'quit)
              (apply quit args))
-        (if (eq? key 'srfi-34)
-            (apply throw key args)                ;handled by callers
-            (report-exception the-action obj key args))))))
+
+        ;; Re-throw SRFI-34 exceptions that the caller will handle.
+        (cond ((eq? key 'srfi-34)                 ;Guile 2.x
+               (apply throw key args))
+              ((eq? key '%exception)              ;Guile 3.x
+               (raise-exception (car args)))
+              (else
+               (report-exception the-action obj key args)))))))
 
 ;; Display documentation about the service.
 (define-method (doc (obj <service>) . args)
@@ -707,10 +714,10 @@ results."
                                passwd:dir)
                         (getcwd)))))
 
-(define (default-environment-variables)
-  "Return the list of environment variable name/value pairs that should be
-set when starting a service."
-  (environ))
+(define default-environment-variables
+  ;; The default list of environment variable name/value pairs that should be
+  ;; set when starting a service.
+  (make-parameter (environ)))
 
 (define* (read-pid-file file #:key (max-delay 5)
                         (validate-pid? #f))
@@ -931,7 +938,6 @@ start."
                     (apply throw args))))))
 
           (clean-up pid-file)
-          (clean-up log-file)
 
           (let ((pid (fork+exec-command command
                                         #:user user
